/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "kmlogprops.h"
#include <kmessagebox.h>

KmLogProps::KmLogProps( QWidget* parent, KmFile *pFile ): 
		QDialog(parent), Ui::KmLogProps()
{
	setupUi( this );
	mpFile = pFile;
	
	QStringList presets, patterns, templates;
	
	presets << "Syslog" << "Emerge" << "MySQL" << "ISO8061" << "KNotify" << "Nagios" 
					<< "Apache/CUPS" << "Samba" << "Other";
	patterns 	<< "^(\\w\\w\\w\\s.{11})\\s" << "^(\\d{10}):\\s" << "^(\\d*\\s.\\d:\\d\\d:\\d\\d)\\s" 
						<< "^(.{23})\\s"<< "^.{13}\\s(\\w\\w\\w\\s.?\\d\\s.{13}):\\s" << "^\\[(\\d{10})\\]\\s"
						<< "\\[(.*)\\s.\\d\\d\\d\\d\\]\\s" << "^\\[(.{19}),.*\\]\\s" << "";
	templates << "MMM d hh:mm:ss" << "ssssssssss" << "yyMMdd h:mm:ss" 
						<< "yyyy-MM-dd hh:mm:ss,zzz" << "MMM d hh:mm:ss yyyy" << "ssssssssss" 
						<< "dd/MMM/yyyy:hh:mm:ss" << "yyyy/MM/dd hh:mm:ss" << "";
	mFieldCnts << 3 << 1 << 1 << 1 << 1 << 1 << 4 << 1 << 1;
	
	mPresets->clear();
	for ( int k = 0; k < presets.count(); ++k )
	{
		mPresets->addItem( presets[k], QStringList() << patterns[k] << templates[k] );
	}	

	mName->setText( pFile->mName );
	
	if ( !pFile->mNoTmplt )
	{
		mTsEnable->setChecked( true );
		slotEnableTmplt();
		
		int k = patterns.indexOf( pFile->mPattern );
		if ( k == -1 ) 
		{
			k = presets.count() - 1;                       // pattern is "Other"
			mPresets->setCurrentIndex( k );
			mPattern->setText( pFile->mPattern );
			mTemplate->setText( pFile->mTemplate );
		}
		else
			mPresets->setCurrentIndex( k );
	}	

	mSpinBox->setValue( pFile->mMsgFieldCnt );
	
	setWindowTitle( pFile->mFileNm );
	mPresets->setToolTip( i18n("Predefined templates for common log types") );
	mPattern->setToolTip( i18n("A regular expression to extract the time-stamp from a log record. "
														 "For more information use <b>What's This</b>") );
	mTemplate->setToolTip( i18n("A QDateTime format to parse the time-stamp. "
														  "For more information use <b>What's This</b>") );
	
	mPattern->setWhatsThis( i18n(
		"<h3>Parsing a time-stamp using a Regular Expression.</h3>"
		"Non-standard format log files may now have their time-stamps extracted and converted "
		"into the standard syslog format. This will usually greatly increase legibility." 
		" To do this requires specifying a regular expression (regexp) "
		"to extact the date/time field which must contain a <b>capture group</b> to delimit the "
		"actual date/time characters (as opposed to any layout). The presets provided should give "
		"you the general idea and can be used as a starter for other log formats.<p> "
		"For more information see: <b>http://doc.qt.nokia.com/4.6/qregexp.html#details</b> for details."
		"of the Qt regexp engine"
		) );
	
	mTemplate->setWhatsThis( i18n(
		"<h3>Specifying a date/time template.</h3>"
		"Once the date/time stamp has been extracted, a template must be specified so that Qt can "
		"convert it into a standard internal format. This will usually be something like: "
		"<b>yyyy-MM-dd hh:mm:ss</b> or some simple variation. Note there is a special template "
		"<b>ssssssssss</b> which means a 10 digit number of seconds and is interpreted as Unix time "
		"(the number of seconds since 1970-01-01 00:00:00). <p>"
		"For more information see: <b>http://doc.qt.nokia.com/4.6/qdatetime.html#fromString-2</b>"
		) );
}

KmLogProps::~KmLogProps()
{
}

void KmLogProps::slotEnableTmplt()
{
	if ( mTsEnable->isChecked() )
	{
		mPattern->setEnabled( true );
		mPresets->setEnabled( true);
		mTemplate->setEnabled( true );
		
		slotPresetChanged( 0 );
	}	
	else
	{
		mPattern->setEnabled( false );
		mPresets->setEnabled( false );
		mTemplate->setEnabled( false );
		mPattern->clear();
		mTemplate->clear();
		mpFile->mPattern.clear();
		mpFile->mTemplate.clear();
	}	
}

void KmLogProps::slotBtnClicked( QAbstractButton *pButton )
{
	if ( mButtonBox->buttonRole( pButton ) == QDialogButtonBox::AcceptRole )
	{
		if ( mName->text().isEmpty() )
		{
			mName->setFocus();
			return;
		}	
		else
			mpFile->mName = mName->text();
		
		mpFile->mMsgFieldCnt = mSpinBox->value();
		
		if ( mTsEnable->isChecked() )
		{
			QRegExp rx( mPattern->text() );
			if ( rx.isValid() )
				mpFile->mPattern = mPattern->text();
			else
			{	
				KMessageBox::sorry( this, QString(i18n("Error in regexp: %1").arg(rx.errorString())) );
				mPattern->setFocus();
				return;
			}	
			
			if ( !mTemplate->text().isEmpty() )
				mpFile->mTemplate = mTemplate->text();
			else
			{
				mTemplate->setFocus();
				return;
			}	
		}
		
		accept();
	}	 
	else
	{
		reject();
	}
}

void KmLogProps::slotPresetChanged( int k )
{
	QStringList data( mPresets->itemData( k ).toStringList() );
	if ( data.isEmpty() ) return;
	
	mPattern->setText( data[0]  );
	mTemplate->setText( data[1] );
	mSpinBox->setValue( mFieldCnts[k] );
}

void KmLogProps::slotSpinBoxChanged( int )
{ // not currently used
}

#include "kmlogprops.moc"