/*
 * kmtail.h
 *
 * Copyright (C) 2008 %{AUTHOR} <%{EMAIL}>
 */
#ifndef KMTAIL_H
#define KMTAIL_H

#define TAIL "turbotail"
#define MAXLINES 1000
#define STD_DATE_FMT "MMM d hh:mm:ss"

#define DEBUG_COL   "#FFFFFF"
#define INFO_COL    "#D2DEF0"
#define NOTICE_COL  "#33FF33"
#define WARNING_COL "#FFFF00"
#define ERR_COL     "#FFAA00"
#define CRIT_COL    "#FF0000"
#define ALERT_COL   "#9999FF"
#define EMERG_COL   "#FF00FF"
#define TSTMP_COL   "#96D2FF"

#include <qtreewidget.h>
#include <qmenu.h>
#include <qbrush.h>
#include <QKeyEvent>
#include <QFileSystemWatcher>
#include <qlabel.h>

#include <kaction.h>
#include <klineedit.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kprocess.h>
#include <kxmlguiwindow.h>
#include <kconfig.h>
#include <kconfiggroup.h>
#include <kcolorbutton.h>
#include <ktabbar.h>
#include <kdirwatch.h>

#include "kmfile.h"
#include "kmtailview.h"


/**
 * This class serves as the main window for KmTail.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author %{AUTHOR} <%{EMAIL}>
 * @version %{VERSION}
 */
class KmTail : public KXmlGuiWindow
{
		Q_OBJECT
		
public:
    /**
     * Default Constructor
     */
    KmTail( KCmdLineArgs *pArgs = 0 );

    /**
     * Default Destructor
     */
    virtual ~KmTail();
		
		friend class KmConf;
		
		enum { DEBUG = 0, INFO = 1, NOTICE = 2, WARNING = 3, ERR = 4, CRIT = 5, ALERT = 6, EMERG = 7 };
		
public slots:

protected:
		void dropEvent( QDropEvent* event );
		void dragEnterEvent( QDragEnterEvent* event );
		bool queryClose();
	
	
private slots:
    void fileOpen();
    void fileClose();
		void closeTab( int );
		void detachTab();
		void changedTab( int );
		void fileReload( KmFile* pFile = 0 );
		void menuHide();
		void rmbItem( const QPoint& );
		void rmbTab( int, const QPoint& );
		void selectAllItems( KmFile* );
		void copySelection( KmFile* );
		void editDone( QString arg );
		void clearEdit();
		void moveElems( int, int );
		void confOptions();
		void fsChange( const QString &path ); 
		void checkFileExists();
		void editProps();
		//void quitting();
	
private:
    void setupActions();
    void addFile(const QString& lf);
		void readFile(KmFile* pFile);
		void saveOptions();
		void readOptions();
		bool highLightItem( QTreeWidgetItem* pItem, QStringList& tags, QString &col, const QString& prio ) const;
		void startFile( KmFile* );
		void stopTail( KmFile* );
		void splitToolTip( QString& );
    void disablePipeNotify();
		bool formatFileLine( QTreeWidgetItem* pItem, QString& line, const KmFile* );
    void skipLines(KmFile* pFile);
    QString fileNm( const QString& );
		void extractDateTime( QTreeWidgetItem*, KmFile*, QString& );
		void parseFields( QTreeWidgetItem* pItem, const KmFile* pFile, QString& line );
		
private:
    KmTailView *mpView;
		KmFileList *mpFiles;
		QString mArgFile;
		QFileSystemWatcher *mpFSW;
    KSharedConfigPtr mConfig;
		KConfigGroup *mpGenGroup;
		QLabel* mpSBtext1;
		KAction *mpFileReload;
		KAction *mpMenuHide;
		KAction *mpFont;
		KAction *mpEdit;
		KAction *mpEditIcon;
		KAction *mpProperties;
		KLineEdit *mpLnEdit;
		QMenu *mpPopup;
		QMenu *mpTabPopup;
		QFont mFont;
		bool mUsrFont;
		QString mTail;
		const QString fTB;
		QList <QStringList> mDefTags;
		QList <QStringList> mTags;
		QStringList mDefPriCols;
		QStringList mPriCols;
		QStringList mPrioTags;
		QStringList mFaclTags;
		QString mTstmpCol;
		QString mResidual;
		int mTabClickedIndex;
		
		QFile *mpLogger;
		QDebug *mpDebug;
};


#endif // _KMTAIL_H_
